### Encoding: UTF-8

### Name: nls.lm
### Title: Addresses NLS problems with the Levenberg-Marquardt algorithm
### Aliases: nls.lm
### Keywords: nonlinear optimize regression

### ** Examples

f <- function(T, tau, N0, a, f0) {
    expr <- expression(N0*exp(-T/tau)*(1 + a*cos(f0*T)))
    eval(expr)
}
j <- function(T, tau, N0, a, f0) {
    expr <- expression(N0*exp(-T/tau)*(1 + a*cos(f0*T)))
    c(eval(D(expr, "tau")),
      eval(D(expr, "N0" )),
      eval(D(expr, "a"  )),
      eval(D(expr, "f0" )))
}

T <- seq(0, 8, len=501)
p <- c("tau" = 2.2, "N0" = 1000, "a" = 0.25, "f0" = 8)

N <- do.call("f", c(list(T = T), as.list(p)))
N <- rnorm(length(N), mean=N, sd=sqrt(N))

plot(T, N, bg = "black", pch = 21, cex = 0.5)

fcn     <- function(p, T, N, N.Err, fcall, jcall)
    (N - do.call("fcall", c(list(T = T), as.list(p))))/N.Err
fcn.jac <- function(p, T, N, N.Err, fcall, jcall) {
    N.Err <- rep(N.Err, length(p))
    -do.call("jcall", c(list(T = T), as.list(p)))/N.Err
}

guess <- c("tau" = 2.2, "N0" = 1500, "a" = 0.25, "f0" = 10)

out <- nls.lm(par = guess, fn = fcn, #jac = fcn.jac,
              fcall = f, jcall = j,
              T = T, N = N, N.Err = sqrt(N), 
              control = nls.lm.control(nprint=1, diag = numeric()))

N1 <- do.call("f", c(list(T = T), out$par))  # N1 == N - sqrt(N)*out$fvec

lines(T, N1, col="blue", lwd=2)

summary(out) 



